/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { IFluidLoadable } from "../../index.js";
// test implicit conversions between FluidObject and a FluidObject with a provides interface
{
    const provider = getFluidObject();
    useFluidObject(provider);
    useFluidObject(provider.IFluidLoadable);
    useProvider(provider);
    useProvider(provider.IFluidLoadable);
    useLoadable(provider);
    useLoadable(provider.IFluidLoadable);
    // @ts-expect-error provider shouldn't have any non-provider properties
    use(provider.handle);
    use(provider.IFluidLoadable?.handle);
    const unknown = provider.IFluidLoadable;
    useFluidObject(unknown);
    useProvider(unknown);
    useProvider(unknown);
    useLoadable(unknown);
}
// test implicit conversions between FluidObject and a FluidObject with a implementation interface
{
    const foo = getFluidObject();
    useFluidObject(foo);
    useFluidObject(foo.IFluidLoadable);
    useProvider(foo);
    useProvider(foo.IFluidLoadable);
    useLoadable(foo);
    useLoadable(foo.IFluidLoadable);
    // @ts-expect-error provider shouldn't have any non-provider properties
    use(foo.handle);
    use(foo.IFluidLoadable?.handle);
    const unknown = foo.IFluidLoadable;
    useFluidObject(unknown);
    useProvider(unknown);
    useProvider(unknown);
    useLoadable(unknown);
}
// test getting keys
{
    useProviderKey(IFluidLoadable);
    useProviderKey(IFluidLoadable);
    const loadableKey = "handle";
    // @ts-expect-error provider shouldn't have any non-provider properties
    useProviderKey(loadableKey);
}
// test implicit conversions between FluidObject and a FluidObject with a partial provider interface
{
    const foo = getFluidObject();
    useFluidObject(foo);
    useFluidObject(foo.IFoo);
    useProvider(foo);
    useProvider(foo.IFoo);
    foo.IFoo?.doFoo();
    const fooKey = "doFoo";
    // @ts-expect-error provider shouldn't have any non-provider properties
    useProviderKey(fooKey);
    const unknown = foo.IFoo;
    useFluidObject(unknown);
    useProvider(unknown);
    useProvider(unknown);
    useLoadable(unknown);
}
// validate nested property is FluidObject too
{
    const foo = getFluidObject();
    // @ts-expect-error "Property 'z' does not exist on type 'FluidObject<IFoo>'."
    useProvider(foo.z);
}
// validate provider inheritance
{
    const p = getFluidObject();
    useProvider(p.IFooParent?.parent());
    const c = getFluidObject();
    // @ts-expect-error Property 'IFooParent' does not exist on type 'FluidObject<IFooProvideChild>'.
    useProvider(c.IFooParent?.parent());
    useProvider(c.IFooChild?.child());
    useProvider(c.IFooChild?.parent());
    useProvider(c.IFooChild?.IFooParent?.parent());
}
// validate usage as builder
{
    const builder = {};
    builder.IFluidLoadable = getLoadable();
}
// validate readonly prevents modification
{
    const builder = {};
    // @ts-expect-error Cannot assign to 'IFluidLoadable' because it is a read-only property.
    builder.IFluidLoadable = getLoadable();
}
//# sourceMappingURL=fluidObjectTypes.js.map